from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowMergeRequest")


@_attrs_define
class WorkflowMergeRequest:
    """Schema for merging workflows

    Attributes:
        target_workflow_id (UUID): Workflow ID to merge into
        copy_trajectories (bool | Unset): Whether to copy trajectories to the target workflow Default: False.
    """

    target_workflow_id: UUID
    copy_trajectories: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        target_workflow_id = str(self.target_workflow_id)

        copy_trajectories = self.copy_trajectories

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "target_workflow_id": target_workflow_id,
            }
        )
        if copy_trajectories is not UNSET:
            field_dict["copy_trajectories"] = copy_trajectories

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        target_workflow_id = UUID(d.pop("target_workflow_id"))

        copy_trajectories = d.pop("copy_trajectories", UNSET)

        workflow_merge_request = cls(
            target_workflow_id=target_workflow_id,
            copy_trajectories=copy_trajectories,
        )

        workflow_merge_request.additional_properties = d
        return workflow_merge_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
